/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.productionunit;

import java.awt.Color;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.Icon;
import nl.cloudfarming.client.farm.model.CropProductionUnit;
import nl.cloudfarming.client.farm.model.CropProductionUnitManager;
import nl.cloudfarming.client.geoviewer.BaseLayer;
import nl.cloudfarming.client.geoviewer.Palette;
import nl.cloudfarming.client.model.EntitySavable;
import org.netbeans.api.actions.Savable;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataObjectExistsException;
import org.openide.loaders.MultiDataObject;
import org.openide.loaders.MultiFileLoader;
import org.openide.nodes.Node;
import org.openide.util.ImageUtilities;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;
import org.openide.util.lookup.AbstractLookup;
import org.openide.util.lookup.InstanceContent;
import org.openide.util.lookup.ProxyLookup;

@NbBundle.Messages("Services/MIMEResolver/CropProductionUnitResolver.xml=CropProductionUnit Files")
public class CropProductionUnitDataObject extends MultiDataObject implements PropertyChangeListener {

    private static final Logger LOGGER = Logger.getLogger("nl.cloudfarming.client.cropfield.productionunit");
    private InstanceContent ic;
    private Lookup intern;
    private CropProductionUnitManagerImpl cropProductionUnitManager;
    private CropProductionUnit cropProductionUnit;

    public CropProductionUnitDataObject(FileObject pf, MultiFileLoader loader) throws DataObjectExistsException, IOException {
        super(pf, loader);
        registerEditor(CropProductionUnit.MIME_TYPE, true);

        this.ic = new InstanceContent();
        this.intern = new AbstractLookup(this.ic);

        cropProductionUnitManager = Lookup.getDefault().lookup(CropProductionUnitManagerImpl.class);
        assert cropProductionUnitManager != null : "No CropProductionManager class found in the default lookup, unable to load the DataObject";

        cropProductionUnit = cropProductionUnitManager.load(getPrimaryFile());
    }

    @Override
    protected int associateLookup() {
        return 1;
    }

    @Override
    protected Node createNodeDelegate() {
        assert (cropProductionUnit != null);
        cropProductionUnit.addPropertyChangeListener(this);

        ic.add(new BaseLayer(new Palette(Color.PINK), cropProductionUnit.getName()));
        ic.add(cropProductionUnit);

        Node delegate = new CropProductionUnitNode(super.createNodeDelegate(), cropProductionUnit);
        return delegate;
    }

    @Override
    public Lookup getLookup() {
        return new ProxyLookup(super.getLookup(), intern);
    }

    /**
     * <p>Listens to changes on {@link CropProductionUnit}</p> <p>If changes are
     * made will add {@link Savable} to the lookup</p>
     *
     * @param evt
     */
    @Override
    public void propertyChange(PropertyChangeEvent evt) {
        Savable savable = this.getLookup().lookup(Savable.class);
        if (null == savable) {
            this.ic.add(new CropProductionUnitSavable());
        }
    }

    /**
     * Looks up all the {@link CropProductionUnitSavable} in the lookup of this
     * DataObject and calls destroy on them.
     *
     * @throws IOException
     */
    @Override
    protected void handleDelete() throws IOException {
        super.handleDelete();
        for (CropProductionUnitSavable savable : getLookup().lookupAll(CropProductionUnitSavable.class)) {
            savable.destroy();
        }
    }

    private void save() {
        CropProductionUnitManager manager = Lookup.getDefault().lookup(CropProductionUnitManager.class);
        if (manager != null) {
            manager.save(cropProductionUnit);
        } else {
            LOGGER.log(Level.WARNING, "CropProductionUnitManager not found in lookup! Could not save CropProductionUnit");
        }
    }
    private static final Icon ICON = ImageUtilities.loadImageIcon("nl/cloudfarming/client/productionunit/green16.png", true);

    /**
     * Savable for CropProductionUnitDataObject
     */
    private class CropProductionUnitSavable extends EntitySavable {

        private CropProductionUnitSavable() {
            super(cropProductionUnit, ICON);
            register();
        }

        @Override
        protected String findDisplayName() {
            return cropProductionUnit.getName();
        }

        @Override
        protected void handleSave() throws IOException {
            dataObject().save();
            destroy();
        }

        /**
         * Unregisters itself and removes itself from the instance content
         */
        @Override
        public void destroy() {
            dataObject().ic.remove(CropProductionUnitSavable.this);
            unregister();
        }

        @Override
        public void dispose() {
            dataObject().dispose();
        }

        CropProductionUnitDataObject dataObject() {
            return CropProductionUnitDataObject.this;
        }

        @Override
        public boolean equals(Object obj) {
            if (obj instanceof CropProductionUnitSavable) {
                CropProductionUnitSavable m = (CropProductionUnitSavable) obj;
                return dataObject() == m.dataObject();
            }
            return false;
        }

        @Override
        public int hashCode() {
            return dataObject().hashCode();
        }
    }
}
