/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.productionunit;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import nl.cloudfarming.client.farm.model.CropProductionUnit;
import nl.cloudfarming.client.util.swing.ASBodyPanel;
import org.openide.util.ImageUtilities;
import org.openide.util.NbBundle;

/**
 * <p>Controller for the {@link CropProductionUnitDetailsPanel} Is able to
 * validate the panel</p> <p>Will listen to changes on the
 * {@link CropProductionUnit} got via
 * {@link CropProductionUnitDetailsController#detailsPanel} {@link CropProductionUnitDetailsPanel#getCropProductionUnit()}
 * And listens to the {@link CropProductionUnitDetailsPanel#getNameField()} for
 * name changes. On change will call
 * {@link CropProductionUnitDetailsController#cropProductionUnitDetailFieldsUpdated()}</p>
 *
 * @author Merijn Zengers
 */
@NbBundle.Messages({
    "CropProductionUnitDetailsController title=Production unit details",
    "CropProductionUnitDetailsController invalid name error=Invalid name",
    "CropProductionUnitDetailsController duplicate name error=Name is not unique",
    "CropProductionUnitDetailsController start date required=Start date is required",
    "CropProductionUnitDetailsController end date required=End date is required",
    "CropProductionUnitDetailsController end date invalid=Invalid end date",
    "CropProductionUnitDetailsController crop definition required=Crop is required",
    "CropProductionUnitDetailsController planned area in ha number=Value must be a number"})
public abstract class CropProductionUnitDetailsController<T extends CropProductionUnitDetailsPanel> {

    protected T detailsPanel;
    private ASBodyPanel component;
    private static final String ICON = "nl/cloudfarming/client/productionunit/green32.png";

    public CropProductionUnitDetailsController(T cropProductionUnitDetailsPanel) {
        this.detailsPanel = cropProductionUnitDetailsPanel;
        detailsPanel.getCropProductionUnit().addPropertyChangeListener(new CropProductionUnitPropertyChangeListener());
        //Add separate Listener for name. This is because empty names are not committed to the model object 
        detailsPanel.getNameField().getDocument().addDocumentListener(new NameChangeListener());
    }

    public ASBodyPanel getComponent() {
        if (component == null) {
            component = new ASBodyPanel(detailsPanel);
            component.setTitle(Bundle.CropProductionUnitDetailsController_title());
            component.setIcon(ImageUtilities.loadImageIcon(ICON, false));
            component.setName(Bundle.CropProductionUnitDetailsController_title());
        }
        return component;
    }

    /**
     * <p>returns if the panel is valid</p> <p> Validates if the name provided
     * is not empty</br> Validates if the start date is set</br> Validates if
     * the end date is set</br> Validates if the start date is before the end
     * date </br> Validates if the planned area is empty or a number</p>
     *
     *
     * @return
     */
    public boolean isPanelValid() {
        return isNameValid() && isStartDateValid() && isEndDateValid() && isStartDateBeforeEndDate() && isPlannedAreaValid();
    }

    /**
     * <p>Validates the panel</p> <p> Validates if the name provided is not
     * empty</br> Validates if the start date is set</br> Validates if the end
     * date is set</br> Validates if the start date is before the end date </br>
     * Validates if the planned area is empty or a number</p>
     *
     * <p>If one of the checks fails sets an error on the field</p>
     *
     */
    public void validatePanel() {
        validateName();
        validateStartDate();
        validateEndDate();
        validatePlannedArea();
    }

    private boolean isNameValid() {
        String userInput = detailsPanel.getNameField().getText();
        if (userInput == null || userInput.isEmpty()) {
            return false;
        }
        return true;
    }

    private boolean isStartDateValid() {
        CropProductionUnit cropProductionUnit = detailsPanel.getCropProductionUnit();
        if (cropProductionUnit.getStartDate() == null) {
            return false;
        }
        return true;
    }

    private boolean isEndDateValid() {
        CropProductionUnit cropProductionUnit = detailsPanel.getCropProductionUnit();
        if (cropProductionUnit.getEndDate() == null) {
            return false;
        }
        return true;
    }

    /**
     * <p>Tests if the planned area is valid</p> The planned area is valid if it
     * is empty or null or is a double
     *
     * @return
     */
    private boolean isPlannedAreaValid() {
        String userInput = detailsPanel.getPlannedAreaField().getText();
        if (userInput != null && !userInput.isEmpty()) {
            try {
                Double.parseDouble(userInput);
            } catch (NumberFormatException ex) {
                return false;
            }
        }
        return true;
    }

    private boolean isStartDateBeforeEndDate() {
        CropProductionUnit cropProductionUnit = detailsPanel.getCropProductionUnit();
        if (cropProductionUnit.getEndDate() != null && cropProductionUnit.getStartDate() != null) {
            if (cropProductionUnit.getEndDate().before(cropProductionUnit.getStartDate())) {
                return false;
            }
        }
        return true;
    }

    private void validateName() {
        if (!isNameValid()) {
            detailsPanel.getNameRow().setInvalid(Bundle.CropProductionUnitDetailsController_invalid_name_error());
        } else {
            detailsPanel.getNameRow().setValid();
        }
    }

    private void validateStartDate() {
        if (!isStartDateValid()) {
            detailsPanel.getValidityRow().setInvalid(Bundle.CropProductionUnitDetailsController_start_date_required());
        } else if (!isStartDateBeforeEndDate()) {
            detailsPanel.getValidityRow().setInvalid(Bundle.CropProductionUnitDetailsController_end_date_invalid());
        }

    }

    private void validateEndDate() {
        if (!isEndDateValid()) {
            detailsPanel.getValidityRow().setInvalid(Bundle.CropProductionUnitDetailsController_end_date_required());
        } else if (!isStartDateBeforeEndDate()) {
            detailsPanel.getValidityRow().setInvalid(Bundle.CropProductionUnitDetailsController_end_date_invalid());
        }
    }

    /**
     * <p>Validates the planned area row</p>
     * Will set error message on row when invalid
     */
    private void validatePlannedArea() {
        if (!isPlannedAreaValid()) {
            detailsPanel.getPlannedAreaRow().setInvalid(Bundle.CropProductionUnitDetailsController_planned_area_in_ha_number());
        } else {
            detailsPanel.getPlannedAreaRow().setValid();
        }
    }

    private class CropProductionUnitPropertyChangeListener implements PropertyChangeListener {

        @Override
        public void propertyChange(PropertyChangeEvent pce) {
            detailsPanel.getValidityRow().setValid();
            if (pce.getPropertyName().equals(CropProductionUnit.PROP_START_DATE)) {
                validateStartDate();
            } else if (pce.getPropertyName().equals(CropProductionUnit.PROP_END_DATE)) {
                validateEndDate();
            }

        }
    }

    private class NameChangeListener implements DocumentListener {

        @Override
        public void insertUpdate(DocumentEvent de) {
            validateName();
        }

        @Override
        public void removeUpdate(DocumentEvent de) {
            validateName();
        }

        @Override
        public void changedUpdate(DocumentEvent de) {
            validateName();
        }
    }
}
