/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.productionunit;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.EnumSet;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;
import nl.cloudfarming.client.farm.model.CropField;
import nl.cloudfarming.client.farm.model.CropProductionUnit;
import nl.cloudfarming.client.farm.model.CropProductionUnitManager;
import nl.cloudfarming.client.farm.model.FarmManager;
import nl.cloudfarming.client.model.*;
import org.netbeans.api.project.Project;
import org.openide.filesystems.FileObject;
import org.openide.util.Exceptions;
import org.openide.util.Lookup;
import org.openide.util.lookup.ServiceProvider;
import org.openide.util.lookup.ServiceProviders;

/**
 *
 * @author Maciek Dulkiewicz <maciek.dulkiewicz@ordina.nl>
 */
@ServiceProviders({
    @ServiceProvider(service = CropProductionUnitManager.class),
    @ServiceProvider(service = EntityManager.class)
})
public class CropProductionUnitManagerImpl extends AbstractEntityManager<CropProductionUnit> implements CropProductionUnitManager {

    private static final Logger LOGGER = Logger.getLogger(CropProductionUnitManagerImpl.class.getName());
    private static final Set<ItemIdType> SUPPORTED_TYPES = EnumSet.of(ItemIdType.CPU, ItemIdType.CFD);
    public static final String CROP_PRODUCTION_UNIT_DIR = "Production units";

    @Override
    public Set<ItemIdType> getSupportedTypes() {
        return SUPPORTED_TYPES;
    }

    @Override
    public void save(CropProductionUnit cropProductionUnit) {
        FarmManager farmManager = Lookup.getDefault().lookup(FarmManager.class);
        Project project = farmManager.findFarmProject(cropProductionUnit.getURI());
        save(cropProductionUnit, project);
    }    
    
    /**
     * Save a crop production unit to a file inside a specific project
     * structure.
     *
     * @param cropProductionUnit The crop production unit that should be saved
     * @param project The project this crop production unit belongs to
     */
    private void save(CropProductionUnit cropProductionUnit, Project project) {
        FileObject cropProductionUnitDir = getCropProductionUnitDir(project);
        if (cropProductionUnitDir != null) {
            FileObject targetFile = getCropProductionUnitTargetFile(cropProductionUnitDir, cropProductionUnit.getURI(), true);
            try (ObjectOutputStream oos = new ObjectOutputStream(targetFile.getOutputStream())) {
                oos.writeObject(cropProductionUnit);
                addToCache(cropProductionUnit);
            } catch (IOException ex) {
                Exceptions.printStackTrace(ex);
                LOGGER.log(Level.WARNING, "Unable to save crop production unit to file {0}", new Object[]{ex.getMessage()});
            }
        }
    }

    /**
     *
     * @param cropProductionUnitDir
     * @param cropProductionUnit
     * @param create create the file if it doesn't exist
     * @return
     */
    private FileObject getCropProductionUnitTargetFile(FileObject cropProductionUnitDir, AgroURI uri, boolean create) {
        FileObject targetFile = cropProductionUnitDir.getFileObject(uri.getItemIdNumber(), CropProductionUnit.EXT);
        if (targetFile == null && create) {
            try {
                targetFile = cropProductionUnitDir.createData(uri.getItemIdNumber(), CropProductionUnit.EXT);
            } catch (IOException ex) {
                LOGGER.log(Level.WARNING, "Unable to create file in which crop production unit can be saved {0}", new Object[]{ex.getMessage()});
            }
        }
        return targetFile;
    }

    /**
     * Load a serialized crop production unit from a FileObject.
     *
     * @param cropProductionUnitFile The file the crop production unit should be
     * loaded from
     * @return The crop production unit contained by the FileObject or null
     */
    CropProductionUnit load(FileObject cropProductionUnitFile) {
        CropProductionUnit loadedCropProductionUnit = null;
        CropProductionUnit cachedCropProductionUnit = null;

        try (ObjectInputStream ois = new ObjectInputStream(cropProductionUnitFile.getInputStream())) {
            loadedCropProductionUnit = (CropProductionUnit) ois.readObject();
            cachedCropProductionUnit = getCachedInstanceFor(loadedCropProductionUnit);
            if  (cachedCropProductionUnit != loadedCropProductionUnit) {
                return cachedCropProductionUnit;
            }
        } catch (ClassNotFoundException | IOException ex) {
            LOGGER.log(Level.SEVERE, "Unable to load serialized crop production unit from file {0} {1}", new Object[]{cropProductionUnitFile.getName(), ex.getMessage()});
        }
        return loadedCropProductionUnit;
    }

    /**
     * Method to fetch the directory in which the crop production units are
     * saved. A valid project is needed to actually pinpoint the directory.
     *
     * @param project The project this method should look in
     * @return FileObject pointing to the crop production unit directory or null
     */
    public static FileObject getCropProductionUnitDir(Project project) {
        FileObject projectDir = project.getProjectDirectory();
        FileObject cropProductionUnitDir = projectDir.getFileObject(CROP_PRODUCTION_UNIT_DIR);
        if (cropProductionUnitDir == null) {
            try {
                cropProductionUnitDir = projectDir.createFolder(CROP_PRODUCTION_UNIT_DIR);
            } catch (IOException ex) {
                LOGGER.log(Level.WARNING, "Unable to create directory for crop production units {0}", new Object[]{ex.getMessage()});
            }
        }
        return cropProductionUnitDir;
    }

    @Override
    protected CropProductionUnit load(AgroURI uri, Project project) {
        assert uri.getItemIdType() == CropProductionUnit.ITEM_ID_TYPE;
        FileObject cropProductionUnitDir = getCropProductionUnitDir(project);
        if (cropProductionUnitDir != null) {
            FileObject cropProductionUnitFile = getCropProductionUnitTargetFile(cropProductionUnitDir, uri, false);
            if (cropProductionUnitFile != null) {
                return load(cropProductionUnitFile);
            }
        }
        return null;
    }

    @Override
    protected void addToCache(Entity entity) {
        super.addToCache(entity);
        // cascade child entities:
        if (entity instanceof CropProductionUnit) {
            for (CropField cfd : ((CropProductionUnit) entity).getCropFields()) {
                addToCache(cfd);
            }
        }
    }

    @Override
    protected void removeFromCache(Entity entity) {
        super.removeFromCache(entity);
        // cascade child entities:
        if (entity instanceof CropProductionUnit) {
            for (CropField cfd : ((CropProductionUnit) entity).getCropFields()) {
                removeFromCache(cfd);
            }
        }
    }

    @Override
    public void remove(CropProductionUnit entity) {
        //TODO implement remove
        throw new UnsupportedOperationException("Not supported yet.");
    }
}
